/*
 * Pcomm is a public domain telecommunication program for Unix
 * designed to operate similar to the popular MSDOS program, ProComm.
 * ProComm (TM) is copyrighted by Datastorm Technologies, Inc.
 *
 * Emmet P. Gray			US Army, HQ III Corps & Fort Hood
 * ...!ihnp4!uiucuxc!fthood!egray	Attn: AFZF-DE-ENV
 *					Directorate of Engineering & Housing
 *					Environmental Management Office
 *					Fort Hood, TX 76544-5057
 *	Beta release	 7 Feb 88
 *	Release 1.0	12 Mar 88
 */

#include <stdio.h>
#include <signal.h>
#include <curses.h>
#ifndef OLDCURSES
#include <term.h>
#endif /* OLDCURSES */
#include <sys/types.h>
#include <sys/stat.h>
#define MAIN
#include "dial_dir.h"
#include "modem.h"
#include "param.h"
#include "status.h"

#ifdef OLDCURSES
char bp[1024];
#define cbreak crmode
#endif /* OLDCURSES */

struct PARAM *param;
struct DIAL_DIR *dir;
struct STATUS *status;
struct MODEM *modem;
int xmc;
char *null_ptr;

main(argc, argv)
int argc;
char *argv[];
{
	int c, ret_code, i, code, quit();
	char *mytty, *ttyname(), *term, *getenv(), *index, *strdup();
	char *extra_dir, buf[80], message[80];
	struct PARAM *read_param();
	struct DIAL_DIR *read_dir();
	struct STATUS *init();
	struct MODEM *read_modem();
	struct stat stbuf;
	void exit(), error_win(), input_on(), free_ptr();
	extern char *optarg;
#ifdef OLDCURSES
	char *tgetstr(), ocbuf[20];
	char *ocbufptr = ocbuf;
#endif /* OLDCURSES */

	signal(SIGINT, SIG_IGN);
	signal(SIGQUIT, SIG_IGN);
	signal(SIGTERM, quit);
	signal(SIGHUP, quit);

	null_ptr = "";
	index = NULL;
	extra_dir = NULL;
					/* the command line */
	while ((c = getopt(argc, argv, "d:f:")) != EOF) {
		switch (c) {
			case 'd':	/* the extra directory to search */
				extra_dir = strdup(optarg);
				break;
			case 'f':	/* the index into the dialing dir */
				index = strdup(optarg);
				break;
			case '?':	/* default */
				fprintf(stderr, "Usage: pcomm [-d directory] [-f index]\n");
				exit(1);
				break;
		}
	}
					/* get terminal type */
	term = getenv("TERM");
	if (term == NULL || *term == '\0') {
		fprintf(stderr, "Windows not supported (TERM not defined)\n");
		exit(1);
	}
					/* see if terminfo entry exists */
#ifdef OLDCURSES
	ret_code = tgetent(bp, term);
#else /* OLDCURSES */
	setupterm(term, 1, &ret_code);
#endif /* OLDCURSES */
	if (ret_code != 1) {
		fprintf(stderr, "Windows not supported (no terminfo data for '%s')\n", term);
		exit(1);
	}
					/* minimum screen size */
#ifdef OLDCURSES
	if (tgetnum("co") < 80 || tgetnum("li") < 24) {
#else /* OLDCURSES */
	if (columns < 80 || lines < 24) {
#endif /* OLDCURSES */
		fprintf(stderr, "Windows not supported (minimum 80x24 screen required)\n");
		exit(1);
	}
					/* must have cursor movement */
#ifdef OLDCURSES
	if (tgetstr("cm", &ocbufptr) == NULL) {
#else /* OLDCURSES */
	if (cursor_address == NULL) {
#endif /* OLDCURSES */
		fprintf(stderr, "Windows not supported (terminal too dumb)\n");
		exit(1);
	}
					/* load magic cookie variable */
#ifdef OLDCURSES
	xmc = tgetnum("sg");
#else /* OLDCURSES */
	xmc = magic_cookie_glitch;
#endif /* OLDCURSES */
					/* ok... now lets go! */
	initscr();
	nonl();
	cbreak();
	noecho();

	param = (struct PARAM *) NULL;
	modem = (struct MODEM *) NULL;
	dir = (struct DIAL_DIR *) NULL;
					/* show the herald, return status */
	status = init(extra_dir, index);
					/* get 'msgs' status */
	mytty = ttyname(0);
	stat(mytty, &stbuf);
	chmod(mytty, 0600);
	status->msg = stbuf.st_mode & 0777;
					/* read the support files */
	param = read_param();
	dir = read_dir();
	modem = read_modem();
					/* short-cut to dialing window ? */
	code = 0;
	if (index != NULL) {
		for (i=1; i<dir->d_entries+1; i++) {
			if (!strcmp(dir->index[i], index)) {
				dir->q_num[0] = i;
				dir->d_cur = i;
				break;
			}
		}
					/* if match is found */
		if (dir->q_num[0] != -1)
			code = dial_win();
		else {
			sprintf(buf, "Can't find index '%s' in dialing directory file", index);
			sprintf(message, "'%s'", status->d_path);
			error_win(0, buf, message);
		}
		free_ptr(index);
	}
					/* start terminal dialogue */
	terminal(code);
	exit(0);
}

/*
 * Something dreadful happened...  Cleanup the mess we made with the
 * tty driver and release the phone line.
 */

int
quit()
{
	void cleanup();

	cleanup(1);
					/* never returns... */
	return(0);
}

/*
 * Check write permission with the real UID and GID.  Returns a 0 on
 * permission denied, 1 on OK, and 2 on OK-but the file already exists.
 */

int
can_write(file)
char *file;
{
	char *p, path[200], *strcpy(), *strrchr();

	p = strcpy(path, file);
					/* dissect the path component */
	if (p = strrchr(path, '/'))
		*(p++) = '\0';
	else
		strcpy(path, ".");
					/* if it already exists */
	if (!access(file, 0)) {
		if (!access(file, 2))
			return(2);
		return(0);
	}
					/* if path is writable */
	if (!access(path, 2))
		return(1);
	return(0);
}
